#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright 2017, Florent Thiery
'''
Criticality: Normal
Checks that partitions are in conformity with expected norms, and that sufficient free space is available
'''
import subprocess
import sys
import os

YELLOW = '\033[93m'
GREEN = '\033[92m'
RED = '\033[91m'
DEF = '\033[0m'


PATHS = [
    {
        'mount_point': '/',
        'condition': '/var',  # ignore this mount point check if /var is on the same device
        'recommended_types': ('ext4', 'zfs', 'xfs'),
        'min_size_gbytes': 6,
        'reco_size_gbytes': 9,
        'min_available_gbytes': 2,
    },
    {
        'mount_point': '/',
        'condition': '!/var',  # ignore this mount point check if /var is not on the same device
        'recommended_types': ('ext4', 'zfs', 'xfs'),
        'min_size_gbytes': 9,
        'reco_size_gbytes': 14,
        'min_available_gbytes': 4,
    },
    {
        'mount_point': '/var',
        'recommended_types': ('ext4', 'zfs', 'xfs'),
        'min_size_gbytes': 2,
        'reco_size_gbytes': 4,
        'min_available_gbytes': 0.5,
    },
    {
        'mount_point': '/home/msuser/msinstance',
        'recommended_types': ('ext4', 'zfs', 'nfs', 'nfs4', 'xfs'),
        'min_size_gbytes': 5,
        'reco_size_gbytes': 300,
        'min_available_gbytes': 5,
    },
    {
        'mount_point': '/home/skyreach',
        'recommended_types': ('ext4', 'zfs', 'nfs', 'nfs4', 'xfs'),
        'min_size_gbytes': 5,
        'reco_size_gbytes': 9,
        'min_available_gbytes': 2,
    },
    {
        'type': 'memory',
        'min_size_gbytes': 2,
        'reco_size_gbytes': 4,
    },
]


def to_gbytes(size_bytes):
    return int(round(size_bytes / (1024 * 1024 * 1024)))


def get_memory_gbytes():
    memory_gbytes = 0
    with open('/proc/meminfo', 'r') as fo:
        for line in fo:
            if 'MemTotal:' in line:
                memory = line.split('MemTotal:')[1].strip()
                memory_kbytes, unit = memory.split(' ')
                if unit != 'kB':
                    print('Warning, unexpected unit %s.' % unit)
                memory_gbytes = int(round(int(memory_kbytes) / (1024 * 1024)))
    if not memory_gbytes:
        print('Failed to get memory size.')
    return memory_gbytes


def get_path_fs(path):
    # Example of "df" output:
    # Filesystem     Type   1B-blocks        Used   Available
    # /dev/loop2     ext4 52710469632 38253940736 11755397120
    status, output = subprocess.getstatusoutput('df --output="source,fstype,size,avail" -B 1 %s | tail -n 1' % path)
    if status == 0:
        dev, fstype, size, available = output.split()
    else:
        dev = fstype = size = available = None
    # Example of "mount" output:
    # /dev/sdb2 on / type ext4 (rw,relatime,errors=remount-ro)
    status, output = subprocess.getstatusoutput("mount | grep '%s '" % dev)
    if status == 0:
        params = output.split()[-1].strip('()').split(',')
    else:
        params = list()
    return dev, fstype, params, to_gbytes(int(size)), to_gbytes(int(available))


def check_allocation(dev):
    root_dev = os.path.basename(dev)[:3]
    if not root_dev:
        return True
    dev_partitions = list()
    with open('/proc/partitions', 'r') as fo:
        for line in fo:
            if root_dev in line:
                dev_partitions.append(line)

    max_size = 0
    total_size = 0
    for p in dev_partitions:
        major, minor, blocks, name = p.split()
        size = int(blocks) * 512
        if name == root_dev:
            max_size = size
            if root_dev.startswith('md'):
                total_size += size
        else:
            total_size += size
    unallocated = max_size - total_size
    unallocated_gbytes = to_gbytes(unallocated)
    if unallocated_gbytes > 1:
        print('Warning: %s%s GB are unallocated on %s.%s' % (YELLOW, unallocated_gbytes, root_dev, DEF))
        return False
    return True


if __name__ == '__main__':
    error = False
    warning = False
    for part_info in PATHS:
        psize = None
        if part_info.get('mount_point'):
            mount_point = part_info['mount_point']
            if os.path.exists(mount_point):
                mount_point = os.path.realpath(mount_point)
                name = 'Partition of %s' % mount_point
                dev, fstype, params, psize, available = get_path_fs(mount_point)
                condition = part_info.get('condition')
                if condition:
                    subdev = get_path_fs(condition.strip('!'))[0]
                    if subdev == dev and not condition.startswith('!'):
                        print('Skipping check of %s with condition %s.' % (mount_point, condition))
                        continue
                    elif subdev != dev and condition.startswith('!'):
                        print('Skipping check of %s with condition %s.' % (mount_point, condition))
                        continue
                if fstype not in part_info.get('recommended_types'):
                    print('Warning: %s fs type not recommended %s(current: %s, recommended: %s)%s.' % (name, YELLOW, fstype, part_info['recommended_types'], DEF))
                    warning = True
                if 'nfs' not in fstype:
                    warning = not check_allocation(dev)
                if 'acl' in params:
                    print('%sThe device %s is mounted using ACL.%s Please set "noacl" mount parameter in fstab and reboot.' % (RED, dev, DEF))
                    error = True
                min_available_gbytes = part_info.get('min_available_gbytes')
                if min_available_gbytes and available < min_available_gbytes:
                    print('%s has less than %s GB available %s(%s GB available)%s.' % (name, min_available_gbytes, RED, available, DEF))
                    error = True
                else:
                    print('%s has more than %s GB available %s(%s GB available)%s.' % (name, min_available_gbytes, GREEN, available, DEF))
            else:
                print('%s not found, cannot check.' % mount_point)
        elif part_info.get('type') == 'memory':
            name = 'Memory'
            psize = get_memory_gbytes()

        if psize:
            if psize < part_info['min_size_gbytes']:
                print('%s is smaller than the minimum required size %s(%s GB < %s GB)%s.' % (name, RED, psize, part_info['min_size_gbytes'], DEF))
                error = True
            elif psize < part_info['reco_size_gbytes']:
                print('%s is smaller than the recommended size %s(%s GB < %s GB)%s.' % (name, YELLOW, psize, part_info['reco_size_gbytes'], DEF))
                warning = True
            else:
                print('%s is bigger than recommended size %s(%s GB >= %s GB)%s.' % (name, GREEN, psize, part_info['reco_size_gbytes'], DEF))

    if error:
        print('Errors found.')
        code = 1
    elif warning:
        print('Some warnings were found.')
        code = 3
    else:
        print(GREEN + 'All OK.' + DEF)
        code = 0

    sys.exit(code)
